Package: iSEEtree
Version: 1.6.0
Authors@R: 
    c(person(given = "Giulio", family = "Benedetti", role = c("aut", "cre"),
           email = "giulio.benedetti@utu.fi",
           comment = c(ORCID = "0000-0002-8732-7692")),
      person(given = "Ely", family = "Seraidarian", role = c("ctb"),
             email = "ely.seraidarian@utu.fi",
             comment = c(ORCID = "0009-0008-8602-093X")),
      person(given = "Leo", family = "Lahti", role = c("aut"),
             email = "leo.lahti@iki.fi",
             comment = c(ORCID = "0000-0001-5537-637X")))
Title: Interactive visualisation for microbiome data
Description: 
    iSEEtree is an extension of iSEE for the TreeSummarizedExperiment data
    container. It provides interactive panel designs to explore hierarchical
    datasets, such as the microbiome and cell lines.
biocViews: Software, Visualization, Microbiome, GUI, ShinyApps,
        DataImport
License: Artistic-2.0
Encoding: UTF-8
Depends: R (>= 4.4.0), iSEE (>= 2.19.4)
Imports: ape, ggplot2, ggtree, grDevices, methods, miaViz, purrr,
        S4Vectors, shiny, mia, shinyWidgets, SingleCellExperiment,
        SummarizedExperiment, tidygraph, TreeSummarizedExperiment,
        utils
Suggests: biomformat, BiocStyle, knitr, RefManageR, remotes, rmarkdown,
        scater, testthat (>= 3.0.0), vegan
URL: https://github.com/microbiome/iSEEtree
BugReports: https://github.com/microbiome/iSEEtree/issues
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/iSEEtree
git_branch: RELEASE_3_23
git_last_commit: 7bf7989
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:49:51 UTC; biocbuild
Author: Giulio Benedetti [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8732-7692>),
  Ely Seraidarian [ctb] (ORCID: <https://orcid.org/0009-0008-8602-093X>),
  Leo Lahti [aut] (ORCID: <https://orcid.org/0000-0001-5537-637X>)
Maintainer: Giulio Benedetti <giulio.benedetti@utu.fi>
