#' iSEE extension for the TreeSummarizedExperiment container
#'
#' \code{iSEEtree} is an extension of \pkg{iSEE} that provides panels for the
#' \code{\link[TreeSummarizedExperiment:TreeSummarizedExperiment-constructor]{TreeSummarizedExperiment}}
#' container, enabling the interactive visualisation of typical microbiome data.
#' The panel layout of iSEEtree is described in \code{\link{iSEE}}.
#'
#' @examples
#' library(iSEEtree)
#' 
#' @seealso \link[iSEE:iSEE]{iSEE} \link[mia:mia]{mia}
#' 
#' @author Giulio Benedetti \email{giulio.benedetti@@utu.fi}
#' 
#' @docType package
#' @keywords internal
#' @name iSEEtree-package
#' 
"_PACKAGE"
